//********************************************************
// uWatch
// Version 1.0
// Last Update: 10th May 08
// Written for the Microchip C30 Compiler
// Copyright(c) 2008 David L. Jones
// http://www.calcwatch.com
// EMAIL: david@alternatezone.com
// Target Device: PIC24FJ64GA004 (44pin)
//********************************************************

/*********************************************************
This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
**********************************************************/

#include <p24Fxxxx.h>

//define the config fuse bits
_CONFIG1(JTAGEN_OFF & GCP_OFF & GWRP_OFF & BKBUG_OFF & COE_OFF & FWDTEN_OFF & WINDIS_OFF & FWPSA_PR128 & WDTPS_PS1);
//_CONFIG2(IESO_OFF & FNOSC_LPRCDIV & FCKSM_CSECME & OSCIOFNC_ON & IOL1WAY_ON & I2C1SEL_PRI & POSCMOD_NONE);
_CONFIG2(IESO_OFF & FCKSM_CSECME & OSCIOFNC_ON & IOL1WAY_ON & I2C1SEL_PRI & POSCMOD_NONE);

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <i2c.h>
#include <rtcc.h>
#include <timer.h>
#include <uart.h>
#include <pwrmgnt.h>
#include <ports.h>
#include <string.h>
#include "include\i2c_eeprom.h"

#define RevString	"Rev 1.0"

//define all the I/O pins
#define Row1		_RB10
#define Row2		_RB11
#define Row3		_RB12
#define Row4		_RB13
#define Row5		_RA10
#define Row6		_RA7
#define Row7		_RB14
#define Col1		_RB15
#define Col2		_RA0
#define Col3		_RC9
#define Col4		_RA1

#define Key1		17
#define Key2		18
#define Key3		19
#define Key4		10
#define Key5		11
#define Key6		12
#define Key7		3
#define Key8		4
#define Key9		5
#define Key0		16
#define KeyPlus		22
#define KeyMinus	15
#define KeyDiv		1
#define KeyMult		8
#define KeyPoint	9
#define KeyMenu		21
#define KeyEnter	25
#define KeyClear	7
#define KeySign		2
#define KeyEXP		20
#define KeyRCL		24
#define KeyRP		14
#define KeyXY		6
#define KeyLP		13		
#define KeyMode		23

#define LCD_RS		_RA2
#define LCD_EN		_RA3

#define LCD_POWER	_RA8
#define LCD_PWM		_RB2
#define LED_POWER	_RA9
#define LED_PWM		_RB3

#define SDA			_RB9
#define SCL			_RB8
#define PER_POWER	_RC8
#define TXD			_RB6
#define RXD			_RB5
#define IRLED		_RB7

#define KeyDelay		150		//250ms key delay
#define MaxLCDdigits	16

#define OR0	 1			//OR function bit masks
#define OR1	 2
#define OR2	 4
#define OR3	 8
#define OR4	 16
#define OR5	 32
#define OR6	 64
#define OR7	 128
#define OR8	 256
#define OR9	 512
#define OR10 1024
#define OR11 2048
#define OR12 4096
#define OR13 8192
#define OR14 16384
#define OR15 32768

#define AND0  0xFFFE		//AND function bit masks
#define AND1  0xFFFD
#define AND2  0xFFFB
#define AND3  0xFFF7
#define AND4  0xFFEF
#define AND5  0xFFDF
#define AND6  0xFFBF
#define AND7  0xFF7F
#define AND8  0xFEFF
#define AND9  0xFDFF
#define AND10 0xFBFF
#define AND11 0xF7FF
#define AND12 0xEFFF
#define AND13 0xDFFF
#define AND14 0xBFFF
#define AND15 0x7FFF

// define LCD bit port commands
// write to the LATCH register instead of the PORT register 
//   to avoid issues with Read-Modify-Write
#define SetLCD_POWER	LATA=LATA|OR8
#define ClearLCD_POWER	LATA=LATA&AND8
#define SetLED_POWER	LATA=LATA|OR9
#define ClearLED_POWER	LATA=LATA&AND9
#define SetLCD_D4		LATC=LATC|OR4
#define SetLCD_D5		LATC=LATC|OR5
#define SetLCD_D6		LATC=LATC|OR6
#define SetLCD_D7		LATC=LATC|OR7
#define ClearLCD_D4		LATC=LATC&AND4
#define ClearLCD_D5		LATC=LATC&AND5
#define ClearLCD_D6		LATC=LATC&AND6
#define ClearLCD_D7		LATC=LATC&AND7
#define SetLCD_RS		LATA=LATA|OR2
#define ClearLCD_RS		LATA=LATA&AND2
#define SetLCD_EN		LATA=LATA|OR3
#define ClearLCD_EN		LATA=LATA&AND3

#define PeripheralIOON()	LATC=LATC|OR8
#define PeripheralIOOFF()	LATC=LATC&AND8

#define DisableSleepTimer() 	IEC0bits.T1IE = 0		//disable Timer1 interrupt
#define EnableSleepTimer() 		IEC0bits.T1IE = 1		//disable Timer1 interrupt

//#define SDA1			LATB=LATB|OR9
//#define SDA0			LATB=LATB&AND9
#define SCL1			LATB=LATB|OR8
#define SCL0	 		LATB=LATB&AND8

//define keypad rows and columns
// "set" outputs a 0 on the row pin
// "clear sets the row pin to an input, to prevent to keys at once causing a short
#define SetRow1			_TRISB10=0
#define SetRow2			_TRISB11=0
#define SetRow3			_TRISB12=0
#define SetRow4			_TRISB13=0
#define SetRow5			_TRISA10=0
#define SetRow6			_TRISA7=0
#define SetRow7			_TRISB14=0
#define ClearRow1		_TRISB10=1
#define ClearRow2		_TRISB11=1
#define ClearRow3		_TRISB12=1
#define ClearRow4		_TRISB13=1
#define ClearRow5		_TRISA10=1
#define ClearRow6		_TRISA7=1
#define ClearRow7		_TRISB14=1

#define MaxRPNmenuItems	4		//this must match the numbr of lines in the menu below

//global variables for both RPN and Algebraic calculator

#define OPRplus			1		//arbitrary value definitions for algebraic operators
#define OPRminus		2
#define OPRmult			3
#define OPRdiv			4
#define OPRpar1			5		//parentheses
#define OPRpar2			6
#define OPRpar3			7
#define OPRpar4			8
#define OPRpar5			9
#define OPRpar6			10
#define OPRnPr			11
#define OPRnCr			12
#define OPRparra		13
#define OPRxpwry		14
#define OPRrtop			15
#define OPRptor			16

#define PI				3.14159265358979
#define PIstring		"3.14159265358979"

int  CurrentMenu;				// The number of the currently active menu line. 0 to MaxRPNmenuItems
char ExponentIncluded;			//FLAG, TRUE if Exponent has already been entered
char DecimalIncluded;			//FLAG, TRUE if decimal point has already been entered
char MinusIncluded;				//FLAG, TRUE if minus sign already included
char MinusIncludedInExponent;	//FLAG, TRUE if minus sign already included in exponent
char InverseKey;				//FLAG, TRUE if the inverse key has been pressed
char HYPkey;					//FLAG, TRUE if the HYP key has been pressed
char DegreesMode;				//FLAG, TRUE if degres mode is on, otherwise radians
char DisplayXreg[MaxLCDdigits+1];	//holds the value currently being entered by the user, or the Xreg
char DisplayYreg[MaxLCDdigits+1];	//holds the value currently in Yreg
char ValueEntered;				//FLAG, TRUE if value has been entered by using the ENTER key
char MenuMode;					//FLAG, TRUE is the menu if switched on.
int  EnableXregOverwrite;		//FLAG, TRUE if the Xreg will be automatically overwritten on first key press (the ENTER key enables this for example)
double Xreg, Yreg, Zreg, Treg;	//the working registers (Treg not used for Algebraic)
double Yreg1, Zreg1;		//extra working registers for algebraic parentheses
double Yreg2, Zreg2;		//extra working registers for algebraic parentheses
double Yreg3, Zreg3;		//extra working registers for algebraic parentheses
double Yreg4, Zreg4;		//extra working registers for algebraic parentheses
double Yreg5, Zreg5;		//extra working registers for algebraic parentheses
double Yreg6, Zreg6;		//extra working registers for algebraic parentheses

int OperatorXY,  OperatorYZ;		//algebraic operators
int OperatorXY1, OperatorYZ1;
int OperatorXY2, OperatorYZ2;
int OperatorXY3, OperatorYZ3;
int OperatorXY4, OperatorYZ4;
int OperatorXY5, OperatorYZ5;
int OperatorXY6, OperatorYZ6;

double Sreg[10];				//the storage registers. Contents retained when calc mode is exited.

// These are the menu options for the RPN calculator
// one set of menu options for each LCD display line
char CalcMenuLine1[MaxRPNmenuItems][MaxLCDdigits+1]=
{
" 1/x  x^2  SQRT ",
" SIN  COS  TAN  ",
" PI   X^Y  e^x  ",
" REC PLAY  CONV "
};
char CalcMenuLine2[MaxRPNmenuItems][MaxLCDdigits+1]=
{
" LOG 10^x  LOGe ",
" ARC  HYP       ",
" R>P  P>R   //  ",
"  x! 3ROOT  A-F ",
};

char LCDline1[MaxLCDdigits+1];	//holds data for the first LCD display line
char LCDline2[MaxLCDdigits+1];	//holds data for the second LCD display line
char LCDhistory1[MaxLCDdigits+1];	//holds a copy of the LCD data for when the LCD is turned off
char LCDhistory2[MaxLCDdigits+1];	//so the program can restore the LCD to were it was when powered back up

BOOL NextMode;			//TRUE if MODE button switches to next mode. FALSE returns to Time/Date mode.
BOOL TwelveHour;		//TRUE if 12 hour mode
BOOL RPNmode;			//TRUE if RPN mode, otherwise Algebraic

//programming mode variables
BOOL ProgPlay;			//TRUE if keystroke playback mode is on
BOOL ProgRec;			//TRUE if keystroke recording mode is on.
unsigned int MemPointer;	//current EEPROM memory location

rtccTime Time;					//typedef that contains the time
rtccDate Date;					//typedef that contains the date

//***********************************
// Insert all of the code for the I2C bit-bang code
#include "I2C-10.c"
//***********************************


//***********************************
//Delay for num milliseconds
//optimised for 250KHz clock
//timing value is only roughly accurate due to overhead
void DelayMs(int num)
{
	int c,d;
	for(d=1;d<15;d++)
		for(c=0;c<num;c++);
}


//************************************
// set the clock rate to 1MHz 
// Used for UART comms
void Clock1MHz(void)
{
	_RCDIV0=1;
	_RCDIV1=1;
	_RCDIV2=0;
}

//************************************
// set the clock rate to 4MHz 
void Clock4MHz(void)
{
	_RCDIV0=1;
	_RCDIV1=0;
	_RCDIV2=0;
}

//************************************
// set the clock rate to 250KHz. 
// High enough speed to give instant calc response,
// but low enough to keep power consumption low
void Clock250KHz(void)
{
	_RCDIV0=1;
	_RCDIV1=0;
	_RCDIV2=1;
}

//************************************
// set the clock rate to 125KHz 
void Clock125KHz(void)
{
	_RCDIV0=0;
	_RCDIV1=1;
	_RCDIV2=1;
}

//************************************
// set the clock rate to 32KHz 
void Clock32KHz(void)
{
	_RCDIV0=1;
	_RCDIV1=1;
	_RCDIV2=1;
}

//************************************
// resets the sleep timer
#define ResetSleepTimer()	TMR1=0

//***********************************
// Insert the LCD code here
#include "uWatch-LCD.c"
//***********************************

//***********************************
// Display the data string passed
// on top line of the LCD display
void UpdateLCDline1(char s[])
{
	int c;
	strcpy(LCDhistory1,s);		//make a backup copy of the display
	lcd_goto(0);
	lcd_puts(s);
	for (c=strlen(s);c<16;c++)
		lcd_write(' ');		//blank the rest of the line
}

//***********************************
// Display the data string passed
// on bottom line of the LCD display
void UpdateLCDline2(char s[])
{
	int c;
	strcpy(LCDhistory2,s);		//make a backup copy of the display
	lcd_goto(40);				//first charcter on line #2
	lcd_puts(s);
	for (c=strlen(s);c<16;c++)
		lcd_write(' ');		//blank the rest of the line
}

//***********************************
//converts a keypad number key code into a real number 
int	ReturnNumber(unsigned char key)
{
	if (key==Key1) return(1);
	if (key==Key2) return(2);
	if (key==Key3) return(3);
	if (key==Key4) return(4);
	if (key==Key5) return(5);
	if (key==Key6) return(6);
	if (key==Key7) return(7);
	if (key==Key8) return(8);
	if (key==Key9) return(9);
	if (key==Key0) return(0);
	return(0);
}

//***********************************
//Writes a data byte to the 24LC512 memory chip on the I2C bus
void I2CmemoryWRITE(unsigned int address, unsigned char data)
{
	DisableSleepTimer();
	Clock32KHz(); 				//slows things down for the bit-bang I2C
	
	//use bit-bang routine instead of hardware I2C module to avoid nasty silicon BUG in I2C1 module
	I2C_stable();
	I2C_start();
	I2C_sendbyte(0xA0);
	I2C_sendbyte(address>>8);
	I2C_sendbyte(address&0xFF);
	I2C_sendbyte(data);
	I2C_stop();
	I2C_stable();
	
/*
	IdleI2C1();
	I2C1CONbits.SEN=1;			//enable start condition
	while(I2C1CONbits.SEN);		//wait for start condition to end
	I2C1TRN=0xA0;				//transmit the chip address and the WRITE bit
	while(I2C1STATbits.TRSTAT);	//wait for transmit to complete
	I2C1TRN=address>>8;			//send high address byte
	while(I2C1STATbits.TRSTAT);	//wait for transmit to complete
	I2C1TRN=address&0xFF;		//send low address byte
	while(I2C1STATbits.TRSTAT);	//wait for transmit to complete

	I2C1TRN=data;				//send low address byte
	while(I2C1STATbits.TRSTAT);	//wait for transmit to complete
	I2C1CONbits.PEN = 1;   		// initiate Stop on SDA and SCL pins
	while (I2C1CONbits.PEN);	//Wait for Stop

	//now wait for the EEPROM write cycle to complete using Acknowledge Polling
	IdleI2C1();					//wait for bus Idle
	I2C1CONbits.SEN=1;			//enable start condition
	while(I2C1CONbits.SEN);		//wait for start condition to end
	I2C1TRN=0xA0;				//transmit the chip address and the WRITE bit
	while(I2C1STATbits.TRSTAT);	//wait for transmit to complete
	
	IdleI2C1();					//wait for bus idle
	while(I2C1STATbits.ACKSTAT!=0)	//wait until an acknowledge is received from EEPROM
		{
		I2C1CONbits.RSEN = 1;		//Generate Restart		
		while (I2C1CONbits.RSEN);	//Wait for restart	
		I2C1TRN=0xA0;				//transmit the chip address and the WRITE bit
		while(I2C1STATbits.TRSTAT);	//wait for transmit to complete
		IdleI2C1();
	}
	
	I2C1CONbits.PEN = 1;   		// initiate Stop on SDA and SCL pins
	while (I2C1CONbits.PEN);	//Wait for Stop
*/
	
	Clock250KHz();
	EnableSleepTimer();
	return;
}

//***********************************
//Reads a data byte from the 24LC512 memory chip on the I2C bus
unsigned char I2CmemoryREAD(unsigned int address)
{
	unsigned char ch;
	unsigned int a;
	DisableSleepTimer();
	Clock32KHz();				//slows things down for the bit-bang I2C

	//use bit-bang routine instead of hardware I2C module to avoid nasty silicon BUG in I2C1 module
	I2C_stable();
	I2C_start();
	I2C_sendbyte(0xA0);
	I2C_sendbyte(address>>8);
	I2C_sendbyte(address&0xFF);
	I2C_start();
	I2C_sendbyte(0xA1);
	ch=I2C_readbyte();
	I2C_stop();
	I2C_stable();
	
/*	code for the I2C hardware module
	IdleI2C1();
	I2C1CONbits.SEN=1;			//enable start condition
	while(I2C1CONbits.SEN);		//wait for start condition to end
	I2C1TRN=0xA0;				//transmit the chip address and the WRITE bit
	while(I2C1STATbits.TRSTAT);	//wait for transmit to complete
	I2C1TRN=address>>8;			//send high address byte
	while(I2C1STATbits.TRSTAT);	//wait for transmit to complete
	I2C1TRN=address&0xFF;		//send low address byte
	while(I2C1STATbits.TRSTAT);	//wait for transmit to complete
	I2C1CONbits.RSEN=1;			//enable restart condition
	while(I2C1CONbits.RSEN);	//wait for restart condition to end

	I2C1TRN=0xA1;				//transmit the chip address and the READ bit
	while(I2C1STATbits.TRSTAT);	//wait for transmit to complete

    I2C1CONbits.RCEN = 1;		//enable receiver
    while(I2C1CONbits.RCEN);	//wait for byte to be received
	ch=I2C1RCV;
	
	I2C1CONbits.PEN = 1;   		// initiate Stop on SDA and SCL pins
	while (I2C1CONbits.PEN);	//Wait for Stop
*/

	Clock250KHz();
	EnableSleepTimer();
	return(ch);
}

//***********************************
// checks for and returns a keystroke from the EEPROM memory
// only if we are in keystroke mode, otherwise returns 0
// Function assumes that MemPointer has been set correctly
unsigned int KeystrokeReplay(void)
{
	unsigned int key;

	if (ProgPlay==FALSE) return(0);		//keystroke reply not enabled, so exit without any key press
	key=I2CmemoryREAD(MemPointer);		//read the keystroke from EEPROM
	if (key==0)							//check for end of keystroke replay
		{
		ProgPlay=FALSE;					//disable keystroke replay
		return(0);
		}
	MemPointer++;						//increment the EEPROM memory pointer
	return(key);						//return the keystroke from memory
}

//***********************************
// records the keystroke into EEPROM memory
// only if we are in keystroke mode, othrwise does nothing
// Function assumes that MemPointer has been set correctly
void KeystrokeRecord(unsigned char key)
{
	if (ProgPlay==TRUE) return;		//can't record while in play mode

	if (key==KeyMode)					//check for end of record mode
		{	
		I2CmemoryWRITE(MemPointer, 0);	//store a zero to end programming mode
		ProgRec=FALSE;					//switch off keystroke record mode	
		UpdateLCDline1("Program Stored.");
		DelayMs(2000);
		return;	
		}

	I2CmemoryWRITE(MemPointer, key);	//store the keystroke into EEPROM
	MemPointer++;						//increment the EEPROM memory pointer
	return;						
}

//***********************************
// scans the keypad and returns the key code pressed.
// key codes are from 1 to 25 and start from the upper left
// then proceed across in each row.
// only one pass of the keys is made, looping must be done by the calling function
//   this allows the calling function to do stuff while scaning keys
// returns 0 if no key pressed
unsigned int KeyScan(void)
{
	unsigned int k;

	k=KeystrokeReplay();		//check for keystroke programming
	if (k>0) return(k);			//return the keystroke from memory

	ClearRow1;		//set all rows to inputs
	ClearRow2;
	ClearRow3;
	ClearRow4;
	ClearRow5;
	ClearRow6;
	ClearRow7;
	
	k=0;			//return 0 if no key pressed

	// note that two attempts must be made to access the first column due to read-modify-write
	SetRow1;
	if (Col1==0) k=KeyDiv;
	if (Col1==0) k=KeyDiv;	//need to try again for this column		
	if (Col2==0) k=KeyMult;
	if (Col3==0) k=KeyMinus;
	if (Col4==0) k=KeyPlus;
	ClearRow1;
	SetRow2;
	if (Col1==0) k=KeySign;
	if (Col1==0) k=KeySign;
	if (Col2==0) k=KeyPoint;
	if (Col3==0) k=Key0;
	if (Col4==0) k=KeyMode;
	ClearRow2;
	SetRow3;
	if (Col1==0) k=Key7;
	if (Col1==0) k=Key7;
	if (Col2==0) k=Key4;
	if (Col3==0) k=Key1;
	ClearRow3;
	SetRow4;
	if (Col1==0) k=Key8;
	if (Col1==0) k=Key8;
	if (Col2==0) k=Key5;
	if (Col3==0) k=Key2;
	ClearRow4;
	SetRow5;
	if (Col1==0) k=Key9;
	if (Col1==0) k=Key9;		
	if (Col2==0) k=Key6;
	if (Col3==0) k=Key3;
	ClearRow5;
	SetRow6;
	if (Col1==0) k=KeyXY;
	if (Col1==0) k=KeyXY;
	if (Col2==0) k=KeyLP;
	if (Col3==0) k=KeyEXP;
	if (Col4==0) k=KeyRCL;
	ClearRow6;
	SetRow7;
	if (Col1==0) k=KeyClear;
	if (Col1==0) k=KeyClear;
	if (Col2==0) k=KeyRP;
	if (Col3==0) k=KeyMenu;
	if (Col4==0) k=KeyEnter;
	ClearRow7;			

	if (k!=0) 				//key was pressed so lets debounce it
		{
		ResetSleepTimer();	//reset the sleep timer every time a key is pressed
		DelayMs(KeyDelay);	//debounce delay
		SetRow1;
		SetRow2;
		SetRow3;
		SetRow4;
		SetRow5;
		SetRow6;
		SetRow7;
		do; while((Col1==0)||(Col2==0)||(Col3==0)||(Col4==0)); 	//wait for the key to be released
		ClearRow1;
		ClearRow2;
		ClearRow3;
		ClearRow4;
		ClearRow5;
		ClearRow6;
		ClearRow7;			
		if (ProgRec) 
			{
			KeystrokeRecord(k);		//record the keystroke
			if (k==KeyMode) return(0);			//don't return to main menu on MODE key
			}
		}

	return(k);		
}

//***********************************
// Insert all of the Calculator common code here
#include "uWatch-CalcCommon.c"
//***********************************

//***********************************
// Insert all of the RPN Calculator code here
#include "uWatch-RPN.c"
//***********************************

//***********************************
// Insert all of the ALG Calculator code here
#include "uWatch-ALG.c"
//***********************************

//***********************************
// Insert all of the Setup code here
#include "uWatch-SetupMode.c"
//***********************************


//***********************************
// converts 2 digit BCD to integer
unsigned int BCDtoDEC(unsigned int num)
{
	unsigned int ones, tens, temp;
	temp=num;
	tens=(temp&0xF0)>>4;
	temp=num;
	ones=temp&0x0F;
	return((tens*10)+ones);
}

//converts a single digit integer into a char
unsigned char itochar(int i)
{
	switch (i)
		{
		case 0: {return('0'); break;}
		case 1: {return('1'); break;}
		case 2: {return('2'); break;}
		case 3: {return('3'); break;}
		case 4: {return('4'); break;}
		case 5: {return('5'); break;}
		case 6: {return('6'); break;}
		case 7: {return('7'); break;}
		case 8: {return('8'); break;}
		case 9: {return('9'); break;}
		default:{return('*'); break;}
		}
}

//***********************************
// sets the RTCWREN bit to enable writing to the RTCC registers
void RTCCunlock(void)
{
	asm volatile("disi #5");
  	asm volatile("mov #0x55, w7");
  	asm volatile("mov w7, _NVMKEY");
  	asm volatile("mov #0xAA, w8");
  	asm volatile("mov w8, _NVMKEY");
  	asm volatile("bset _RCFGCAL, #13");
  	asm volatile("nop");
  	asm volatile("nop");
}



//***********************************
// switches the LCD power ON
void LCDon(void)
{
	T2CONbits.TON=1;	//turn on timer 2 for PWM
	SetLCD_POWER;
	DelayMs(100);
}

//***********************************
// switches the LCD power OFF
void LCDoff(void)
{
	PORTC=PORTC&0xFF00;			//switch all 8 LCD data pins low.
	ClearLCD_RS;				//switch control lines low
	ClearLCD_EN;
	T2CONbits.TON=0;			//turn off timer 2 for PWM
	ClearLCD_POWER;
}

//***********************************
void IdleI2C1(void)
{
    /* Wait until I2C Bus is Inactive */
    while(I2C1CONbits.SEN || I2C1CONbits.PEN || I2C1CONbits.RCEN || I2C1CONbits.ACKEN || I2C1STATbits.TRSTAT);  
}


//**********************************
// displays the time and date on the LCD
void TimeDateDisplay(void)
{
	char s[MaxLCDdigits+1];
//	char days[8][4]={"   ","Sun","Mon","Tue", "Wed", "Thu", "Fri", "Sat"};
	char months[13][4]={"   ","Jan","Feb","Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
	BOOL pm;
	int temp;

	RtccReadTime(&Time);			//read the RTCC registers
	RtccReadDate(&Date);

	if (Time.f.hour>0x12) pm=TRUE; else pm=FALSE;

	if (TwelveHour==TRUE)
		{	
		switch(Time.f.hour)		//convert from 24hour to 12hour time
			{
			case 0x13: Time.f.hour=0x01; break;
			case 0x14: Time.f.hour=0x02; break;
			case 0x15: Time.f.hour=0x03; break;
			case 0x16: Time.f.hour=0x04; break;
			case 0x17: Time.f.hour=0x05; break;
			case 0x18: Time.f.hour=0x06; break;
			case 0x19: Time.f.hour=0x07; break;
			case 0x20: Time.f.hour=0x08; break;
			case 0x21: Time.f.hour=0x09; break;
			case 0x22: Time.f.hour=0x10; break;
			case 0x23: Time.f.hour=0x11; break;
			case 0x24: Time.f.hour=0x12; break;
			}
		}
	temp=Time.f.hour;
	s[0]=' ';
	s[1]=' ';
	s[2]=' ';
	s[3]=itochar(temp>>4);
	if (s[3]=='0') s[3]=' ';			//remove the leading zero
	s[4]=itochar(Time.f.hour&0x0F);
	s[5]=':';
	temp=Time.f.min;
	s[6]=itochar(temp>>4);
	s[7]=itochar(Time.f.min&0x0F);
	s[8]=':';
	temp=Time.f.sec;
	s[9]=itochar(temp>>4);
	s[10]=itochar(Time.f.sec&0x0F);
	if (TwelveHour==TRUE)
		{
		s[11]='a';
		if (pm) s[11]='p';
		s[12]='m';
		}
	else
		{
		s[11]=' ';
		s[12]=' ';
		}
	s[13]=NULL;
	UpdateLCDline1(s);					//display the time

//	s[0]=days[Date.f.wday][0];
//	s[1]=days[Date.f.wday][1];
//	s[2]=days[Date.f.wday][2];
//	s[3]='-';
	s[0]=' ';
	temp=Date.f.mday;
	s[1]=itochar(temp>>4);
	if (s[1]=='0') s[1]=' ';
	temp=Date.f.mday;
	s[2]=itochar(temp&0x0F);
	s[3]='t';
	s[4]='h';
	if ((Date.f.mday==0x01)||(Date.f.mday==0x21)||(Date.f.mday==0x31))
		{
		s[3]='s';
		s[4]='t';
		}
	if ((Date.f.mday==0x02)||(Date.f.mday==0x22))
		{
		s[3]='n';
		s[4]='d';
		}
	if ((Date.f.mday==0x03)||(Date.f.mday==0x23))
		{
		s[3]='r';
		s[4]='d';
		}

	s[5]=' ';
	temp=BCDtoDEC(Date.f.mon);
	if (temp>12) temp=0;
	s[6]=months[temp][0];
	s[7]=months[temp][1];
	s[8]=months[temp][2];


	s[9]=' ';
	s[10]='2';
	s[11]='0';
	temp=Date.f.year;	
	s[12]=itochar(temp>>4);
	s[13]=itochar(Date.f.year&0x0F);
	s[14]=NULL;
	UpdateLCDline2(s);					//display the date
}

//***********************************
// Turns the backlight ON
void BacklightON(void)
{
	_RP3R=18;				//map RP3 pin 24 to OC1 PWM output using Peripheral Pin Select (for backlight power)
	SetLED_POWER;
}

//***********************************
// Turns the backlight OFF
void BacklightOFF(void)
{
	_RP3R=0;				//map RP3 pin 24 to be an I/O pin
	ClearLED_POWER;
}

//***********************************
// Interrupt service routine for Timer1
// This is called by a timeout because the user as not pressed a key in a few minutes
void __attribute__((__interrupt__)) _T1Interrupt( void )
{
	char s[17];
	T1CONbits.TON=0;			//switch the SLEEP timer OFF
	LCDoff();					//switch OFF LCD power
	BacklightOFF();				//switch off backlight
	I2C_stable();				//disable I2C bus to save power
	_TRISB8=1;			//SCL
	_TRISB9=1;			//SDA
	IFS0bits.T1IF = 0;			//clear Timer1 interrupt
	SetRow2;			//switch on the keypad row so the wake-up key can be activated.
	SetRow6;			//switch on the keypad row so the wake-up key can be activated.
	CNEN1bits.CN3IE=1;	//enable CN3 pin for interrupt in SLEEP mode
	IFS1bits.CNIF=0;	//clear input change interrupt
	IPC4bits.CNIP0=1;	//set CN pin change interrupt priority level
	IPC4bits.CNIP1=0;	//priority is lower than Timer1, so CN vector is not called
	IPC4bits.CNIP2=0;	//operation simply continues from after the sleep instruction
	IEC1bits.CNIE=1;	//enable the pin change interrupt, ready to wake up

	Sleep();							//put the watch to SLEEP.
	//only the RTCC is kept running at this point.
	//when the watch wakes back up, operation will continue from this point.

	IEC1bits.CNIE=0;			//switch off the pin change interrupt
	IFS1bits.CNIF=0;			//clear input change interrupt
	LCDon();					//switch on LCD power
	ResetSleepTimer();			//reset the timer
	lcd_init();					//initialise LCD
	lcd_clear();

	//re-enable the I2C bus
	_TRISB8=0;			//SCL
	_TRISB9=0;			//SDA
	init_I2C();

	strcpy(s,LCDhistory1);		//restore the previous display contents
	UpdateLCDline1(s);			
	strcpy(s,LCDhistory2);
	UpdateLCDline2(s);		
	ResetSleepTimer();
	T1CONbits.TON=1;			//switch the SLEEP timr back on
	DelayMs(500);				//add a key delay
}



//***********************************
void ProgramInit(void)
{
	_ADON=0;		//switch OFF ADC
	AD1PCFG=0xFFFF;	//set all ADC pins to digital I/O

	//define all the I/O pins

	CNPU1=0;		//disable all pullups
	CNPU2=0;

	_CN11PUE=1;		//enable pull-up's on port pins for keypad columns
	_CN2PUE=1;
	_CN19PUE=1;
	_CN3PUE=1;
	
	Clock250KHz();		//set the clock freuqnecy to 250KHz
	
	TRISA=0xFFFF;		//set all pins as inputs
	TRISB=0xFFFF;		//set all pins as inputs
	TRISC=0xFFFF;		//set all pins as inputs

	_TRISC0=0;			//set LCD pins as outputs
	_TRISC1=0;
	_TRISC2=0;
	_TRISC3=0;
	_TRISC4=0;
	_TRISC5=0;
	_TRISC6=0;
	_TRISC7=0;
	_TRISA2=0;
	_TRISA3=0;

	_TRISB6=0;			//TX
	_TRISB7=0;			//IRLED
	_TRISC8=0;			//Peripheral I/O
	_TRISB8=0;			//SCL
	_TRISB9=0;			//SDA

	_TRISA8=0;			//LCD_POWER
	_TRISA9=0;			//LED_POWER

	PORTA=0;			//set all keypad output lines low
	PORTB=0;			//set all keypad output lines low

	//setup Timer1 used for the watch SLEEP timeout
	T1CONbits.TCKPS0=1;		//clock prescaler = 256
	T1CONbits.TCKPS1=1;		//timer ticks over at 128 times per sec
	T1CONbits.TCS=1;		//exteral clock select (32.768KHz crystal)
	IPC0bits.T1IP = 4;		//set Timer1 interrupt priority to 4 
	PR1 = 7680; 			//set the timeout value in the period register
							//7680 = 1 minute
							//128 counts for every second
	TMR1=0;
	T1CONbits.TON=1;		//switch on sleep timer
	IEC0bits.T1IE = 1;		//enable Timer1 interrupt

	//setup Timer2 used for the PWM LCD Display Power & backlight display
	T2CONbits.T32=0;
	T2CONbits.TON=1;
	T2CONbits.TCKPS0=0;		// /DIV prescaler closk for Timer2
	T2CONbits.TCKPS1=0;
	PR2=0x0001;
	TMR2=0x0;
	OC1RS=0x01;

	OC1CONbits.OCM0=0;		//switch PWM mode ON on OC1
	OC1CONbits.OCM1=1;
	OC1CONbits.OCM2=1;
	OC1CONbits.OCTSEL=0;	//output compare registers use Timer2
	
	_RP2R=18;				//map RP2 pin 23 to OC1 PWM output using Peripheral Pin Select (for LCD power)


	//initialise the RTCC
  	__builtin_write_OSCCONL(2);		// enable the Secondary Oscillator
	RTCCunlock();			//allow write access to the RTCC
	mRtccOn();				//switch on the RTCC

	//initialise the RTCC with something on first start, just in case it's corrupted
	Date.f.wday=1;
	Date.f.mday=1;
	Date.f.mon=1;
	Date.f.year=1;
	Time.f.hour=1;
	Time.f.min=1;
	Time.f.sec=1;
//	RtccWriteTime(&Time, FALSE);			//write the RTCC registers
//	RtccWriteDate(&Date, FALSE);

	//disable voltage regulator in SLEEP mode to save power
	RCONbits.VREGS=0;

	//UART setup
	_RP6R=3;				//map RP6 pin 42 to UART1 TX function
	RPINR18bits.U1RXR=5;	//map RP5 pin 41 to UART1 RX function
	U1MODEbits.UARTEN=1;	//enable UART
	U1STAbits.UTXEN=1;		//enable transmitter
	U1MODEbits.BRGH=0;		//low baud rate select
	U1BRG=12;				//buad rate = 2400 with BRGH=0 and 1MHz clock
	U1MODEbits.LPBACK=0;

	//I2C setup
//	I2C1BRG=2;				//buad rate reg must be a minimum of 2
	I2C1CONbits.I2CEN=0;	//disable the I2C bus

								// Power Down mode enabled, 11 bit sampling
	TwelveHour=TRUE;		//TRUE if 12 hour mode
	RPNmode=FALSE;			//default to Algebraic mode
	ProgPlay=FALSE;			//turn of keystroke playback
	ProgRec=FALSE;			//turn off keystroke record
	DegreesMode=FALSE;		//default is radians mode

	return;
}


//************************************************
void OpenTimer1(unsigned int config,unsigned int period)
{
    TMR1  = 0;          /* Reset Timer1 to 0x0000 */
    PR1   = period;     /* assigning Period to Timer period register */
    T1CON = config;     /* Configure timer control reg */
}

//***********************************
int main(void)
{
	char Key;
	char s[10];
	char byte,sbyte;
	int c;
	int fail;
	ProgramInit();	//initialse everything

	lcd_init();
	lcd_clear();

	Clock250KHz();
	PeripheralIOOFF();

	init_I2C();

	RCFGCALbits.CAL=I2CmemoryREAD(63535);		//retrieve calibration value from last byte

	BacklightOFF();

	while(TRUE)		//The main endless loop
					//the MODE key switches between apps
		{
		NextMode=TRUE;

		do {
			TimeDateDisplay();
			for(c=0;c<50;c++)
				{
				Key=KeyScan();
				if (Key!=0) break;
				}
			
			if (Key==KeyRCL)
				{
				BacklightON();
				for(c=0;c<3000;c++)
					{
					Key=KeyScan();
					if (Key==KeyRCL) break;		//if RCL key is pressed again within a few seconds
					}
				if (Key!=KeyRCL) BacklightOFF();	//if key was not pressed a second time then swithc off backlight
				}									//otherwise leave the backlight on so user can use other modes
				
			} while (Key!=KeyMode);			//loop until a key has been pressed
	
		if (RPNmode) RPNcalculator(); else ALGcalculator();
		if (NextMode==FALSE) continue;		//user did something in the calc mode, so return to time/date

		SetupMode();
		}
}
