//********************************************************
// uWatch
// Setup Mode Functions
// Version 1.0
// Last Update: 8th May 08
// Copyright(c) 2008 David L. Jones
// Written for the Microchip C30 Compiler
// Target Device: PIC24FJ64GA004 (44pin)
// http://www.calcwatch.com
// EMAIL: david@alternatezone.com

// NOTE: This code is designed to be inserted inline into the uWatch-Main.C function
//       Most variables are passed as globals and are defined in uWatch-Main.C
//********************************************************

//global variables for Setup Mode

/*********************************************************
This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
**********************************************************/


//***********************************
// The main setup mode routine
// Note that all variables are global
void SetupMode(void)
{
	unsigned int Key, KeyPress2;		//keypress variables
	char s[17];
	unsigned int Mode, num;
	unsigned int c,c2;

	Mode=0;

	while(TRUE) 	//main keypress loop is infinite.
		{

		switch(Mode)
		{
			case 0: strcpy(s,"Set Time"); break;
			case 1: strcpy(s,"Set Date"); break;
			case 2: strcpy(s,"Calc Mode"); break;
			case 3: strcpy(s,"Clear EEPROM"); break;
			case 4: strcpy(s,"Self Test"); break;
			case 5: strcpy(s,"LCD timeout"); break;
			case 6: strcpy(s,"ClockCalibration"); break;
			case 7: strcpy(s,"12/24hr Time"); break;
			case 8: strcpy(s,"About"); break;
		}

		UpdateLCDline1(s);
		UpdateLCDline2("SETUP: +/- & ENT");

		do {			//wait for a keypress
			Key=KeyScan();
		} while(Key==0);

		ResetSleepTimer();		
		//start to process the keypress

		if (Key==KeyMode)		// mode key was pressed, exit setup mode
			return;

		NextMode=FALSE;				//user pressed some key other than MODE, so ensure that when exit we go back to the time/date display
		ResetSleepTimer();

		if (Key==KeyPlus)				//user has pressed the PLUS key
			Mode++;
		if (Mode>8) Mode=0;

		if (Key==KeyMinus)				//user has pressed the MINUS key
			Mode--;
		if (Mode<0) Mode=8;

		if (Key==KeyEnter)				//user has pressed the ENTER key
			{
			switch(Mode)				
				{
				case 0:					//change time
					{
					UpdateLCDline2("(Press 2 Digits)");
					UpdateLCDline1("Enter Hours: ");
					do KeyPress2=KeyScan(); while(KeyPress2==0);
					if (KeyPress2==KeyClear) return;
					num=ReturnNumber(KeyPress2);
					do KeyPress2=KeyScan(); while(KeyPress2==0);
					num=(num<<4)+ReturnNumber(KeyPress2);
					//the result is now an 8bit BCD number
					RCFGCALbits.RTCPTR=1;		//select correct RTC register
					RTCVAL=num;

					UpdateLCDline1("Enter Minutes: ");
					do KeyPress2=KeyScan(); while(KeyPress2==0);
					num=ReturnNumber(KeyPress2);
					do KeyPress2=KeyScan(); while(KeyPress2==0);
					num=(num<<4)+ReturnNumber(KeyPress2);
		
					UpdateLCDline1("Enter Seconds: ");
					do KeyPress2=KeyScan(); while(KeyPress2==0);
					num=(num<<4)+ReturnNumber(KeyPress2);
					do KeyPress2=KeyScan(); while(KeyPress2==0);
					num=(num<<4)+ReturnNumber(KeyPress2);
					//the result is now a 16bit BCD number
					RCFGCALbits.RTCPTR=0;		//select correct RTC register
					RTCVAL=num;
					
					return;
					}
				case 1:					//change date
					{
					UpdateLCDline2("(Press 2 Digits)");
					UpdateLCDline1("Enter Month: ");
					do KeyPress2=KeyScan(); while(KeyPress2==0);
					if (KeyPress2==KeyClear) return;
					num=ReturnNumber(KeyPress2);
					do KeyPress2=KeyScan(); while(KeyPress2==0);
					num=(num<<4)+ReturnNumber(KeyPress2);
					//month and day are combined in the one word
					UpdateLCDline1("Enter Day: ");
					do KeyPress2=KeyScan(); while(KeyPress2==0);
					num=(num<<4)+ReturnNumber(KeyPress2);
					do KeyPress2=KeyScan(); while(KeyPress2==0);
					num=(num<<4)+ReturnNumber(KeyPress2);
					//the result is now a 16bit BCD number
					RCFGCALbits.RTCPTR=2;		//select correct RTC register
					RTCVAL=num;

					UpdateLCDline1("Enter Year: ");
					do KeyPress2=KeyScan(); while(KeyPress2==0);
					num=ReturnNumber(KeyPress2);
					do KeyPress2=KeyScan(); while(KeyPress2==0);
					num=(num<<4)+ReturnNumber(KeyPress2);
					//the result is now an 8bit BCD number
					RCFGCALbits.RTCPTR=3;		//select correct RTC register
					RTCVAL=num;
				
					return;
					}
				
				case 2:
					{
					UpdateLCDline2("  +/- & ENTER");
					while(TRUE)
						{
						if (RPNmode) 
							UpdateLCDline1("Calc Mode = RPN");
						else UpdateLCDline1("Calc Mode = ALG");
						do KeyPress2=KeyScan(); while(KeyPress2==0);
						if (KeyPress2==KeyEnter) return;
						if ((KeyPress2==KeyPlus)||(KeyPress2==KeyMinus))
							RPNmode=!RPNmode;
						}
					return;
					}
				
				case 3:					//clear the EEPROM contents
					{
					UpdateLCDline1("Erase EEPROM ?");
					UpdateLCDline2("ENTER or Cancel");
					do KeyPress2=KeyScan(); while(KeyPress2==0);
					if (KeyPress2==KeyEnter)
						{
						UpdateLCDline1("Erasing EEPROM");
						c2=0;
						for(c=0;c<65534;c++)		//don't overwrite the last byte, it's used for the calibration value
							{
							ResetSleepTimer();			//we don't want to timeout while doing this
							if (KeyScan==KeyClear) break;
							I2CmemoryWRITE(c,0);
							c2++;
							if (c2>=100)	
								{
								sprintf(s,"%5i of 65535",c);	
								UpdateLCDline2(s);
								c2=0;
								}
							}
						}
					return;
					}
				
				case 4:
					{
					I2CmemoryWRITE(65530, 0xAA);
					c=I2CmemoryREAD(65530);
					I2CmemoryWRITE(65530, 0);
					c2=I2CmemoryREAD(65530);
					if ((c!=0xAA)||(c2!=0))
						{
						UpdateLCDline1("EEPROM failed!");
						UpdateLCDline2("Press ENTER");
						do KeyPress2=KeyScan(); while(KeyPress2==0);
						return;
						}
					UpdateLCDline1("EEPROM passed");
					UpdateLCDline2("Press ENTER");
					do KeyPress2=KeyScan(); while(KeyPress2==0);
					UpdateLCDline1("Keyboard Test");
					UpdateLCDline2("MODE to exit");
					
					while(TRUE)
						{
						do KeyPress2=KeyScan(); while(KeyPress2==0);
						switch(KeyPress2)
							{
							case Key1: UpdateLCDline2("1"); break;
							case Key2: UpdateLCDline2("2"); break;
							case Key3: UpdateLCDline2("3");  break;
							case Key4: UpdateLCDline2("4"); break;
							case Key5: UpdateLCDline2("5");  break;
							case Key6: UpdateLCDline2("6"); break;
							case Key7: UpdateLCDline2("7");  break;
							case Key8: UpdateLCDline2("8");  break;
							case Key9: UpdateLCDline2("9");  break;
							case Key0: UpdateLCDline2("0");  break;
							case KeyPoint: UpdateLCDline2(".");  break;
							case KeyPlus: UpdateLCDline2("+");  break;
							case KeyMinus: UpdateLCDline2("-"); break;
							case KeyMult: UpdateLCDline2("X"); break;
							case KeyDiv: UpdateLCDline2("/"); break;
							case KeyMenu: UpdateLCDline2("Menu"); break;
							case KeyEnter: UpdateLCDline2("Enter"); break;
							case KeyClear: UpdateLCDline2("Clear"); break;
							case KeySign: UpdateLCDline2("+/-"); break;
							case KeyEXP: UpdateLCDline2("EXP"); break;
							case KeyRCL: UpdateLCDline2("STO/RCL"); break;
							case KeyLP: UpdateLCDline2("(ROLL"); break;
							case KeyRP: UpdateLCDline2(")"); break;
							case KeyXY: UpdateLCDline2("X-Y"); break;
							case KeyMode: return;
							}
						}
					return;
					}

				case 5:
					{
					UpdateLCDline1("+/- Adj Timeout");
					do
						{
						sprintf(s,"%3i Seconds",PR1/128);
						UpdateLCDline2(s);
						do KeyPress2=KeyScan(); while(KeyPress2==0);
						if ((KeyPress2==KeyPlus)&&(PR1<53760)) PR1=PR1+1280;		//increment by 10 seconds
						if ((KeyPress2==KeyMinus)&&(PR1>1280)) PR1=PR1-1280;		//decrement by 10 seconds
						if (KeyPress2==KeyEnter) break;
						}
					while(1);	
					return;
					}

				case 6:
					{
					UpdateLCDline1("+/- Adjust CAL");
					do
						{
						sprintf(s,"CAL=%4i",RCFGCALbits.CAL);
						UpdateLCDline2(s);
						do KeyPress2=KeyScan(); while(KeyPress2==0);
						if (KeyPress2==KeyPlus) RCFGCALbits.CAL++;
						if (KeyPress2==KeyMinus) RCFGCALbits.CAL--;
						if (KeyPress2==KeyEnter) break;
						}
					while(1);	
					I2CmemoryWRITE(63535,RCFGCALbits.CAL);		//store value in last byte
					return;
					}

				case 7:
					{
					UpdateLCDline2("  +/- & ENTER");
					while(TRUE)
						{
						if (TwelveHour) 
							UpdateLCDline1("Time Mode = 12h");
						else UpdateLCDline1("Time Mode = 24h");
						do KeyPress2=KeyScan(); while(KeyPress2==0);
						if (KeyPress2==KeyEnter) return;
						if ((KeyPress2==KeyPlus)||(KeyPress2==KeyMinus))
							TwelveHour=!TwelveHour;
						}
					return;
					}

				case 8:
					{
					strcpy(s," Watch ");
					strcat(s,RevString);
					s[0]=228;				//mu symbol
					UpdateLCDline1(s);
					UpdateLCDline2("(c)David L Jones");
					do KeyPress2=KeyScan(); while(KeyPress2==0);
					return;
					}
				}
			}
		}	//end of entire loop
}

